<?php

/*
 *  @author Pagadito <soporte@pagadito.com>
 *  @copyright  2017 Pagadito
 *
 */

use PrestaShop\PrestaShop\Core\Payment\PaymentOption;

if (!defined('_PS_VERSION_'))
    exit;

class PagaditoPS extends PaymentModule {

    private $_error = array();
    private $_validation = array();

    public function __construct() {
        $this->name = 'pagaditops';
        $this->tab = 'payments_gateways';
        $this->version = '1.2.1';
        $this->author = 'Pagadito';
        $this->display = 'view';
        $this->is_eu_compatible = 1;
        $this->need_instance = 1;
        $this->className = 'PagaditoPS';
        $this->ps_versions_compliancy = array('min' => '1.7', 'max' => _PS_VERSION_);

        $this->currencies = true;
        $this->bootstrap = true;
        parent::__construct();

        $this->displayName = $this->l('Pagadito');
        $this->description = $this->l('Accept payments using Pagadito.');
        $this->confirmUninstall = $this->l('Are you sure you want to delete your details?');
        $this->module_link = $this->context->link->getAdminLink('AdminModules', true).'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;

    }

    /**
     * Pagadito installation process:
     *
     * Step 1 - Requirements checks cURL extension available
     * Step 2 - Pre-set Configuration option values
     * Step 3 - Install the Addon and create a database table to store transaction details
     *
     * @return boolean Installation result
     */
    public function install() {

        /* The cURL PHP extension must be enabled to use this module */
        if (!function_exists('curl_version')) {
            $this->_errors[] = $this->l('Sorry, this module requires the cURL PHP Extension (http://www.php.net/curl), which is not enabled on your server. Please ask your hosting provider for assistance.');
            return false;
        }

        /* General Configuration options */
        Configuration::updateValue('PAGADITO_UID', '');
        Configuration::updateValue('PAGADITO_WSK', '');
        Configuration::updateValue('PAGADITO_SANDBOX', false);

        //installing
        if (!parent::install() || !$this->registerHook('paymentOptions')
            || !$this->registerHook('adminOrder')
            || !$this->registerHook('paymentReturn')
            || !$this->registerHook('displayOrderConfirmation')
            || !$this->_installDb()) {
            return false;
        }

        return true;
    }

    /**
     * Pagadito database table installation (to store the transaction details)
     *
     * @return boolean Database table installation result
     */
    private function _installDb() {
        return Db::getInstance()->Execute('
		CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'pagadito_transaction` (
			`id_pagadito_transaction` int(11) NOT NULL AUTO_INCREMENT,
                        `mode` enum(\'live\',\'sanbox\') NOT NULL,
			`id_shop` int(11) unsigned NOT NULL DEFAULT \'0\',
			`id_customer` int(11) unsigned NOT NULL,
			`id_cart` int(11) unsigned NOT NULL,
			`id_order` int(11) unsigned NOT NULL,
			`amount` decimal(10,2) NOT NULL,
			`currency` varchar(3) NOT NULL,
			`date_add` datetime NOT NULL,
                        `pagadito_token` varchar(255) NULL,
                        `pagadito_transaction_status` varchar(255) NULL,
                        `pagadito_reference` varchar(255)  NULL,
		PRIMARY KEY (`id_pagadito_transaction`), KEY `idx_transaction` (`id_cart`,`id_order`))
		ENGINE=' . _MYSQL_ENGINE_ . ' DEFAULT CHARSET=utf8 AUTO_INCREMENT=1');
    }

    /**
     * Pagadito uninstallation process:
     *
     * Step 1 - Remove Configuration option values from database
     * Step 2 - Remove the database containing the transaction details
     * Step 3 - Uninstallation of the Addon itself
     *
     * @return boolean Uninstallation result
     */
    public function uninstall() {
        $keys_to_uninstall = array('PAGADITO_UID', 'PAGADITO_WSK', 'PAGADITO_SANDBOX');

        $result = true;
        foreach ($keys_to_uninstall as $key_to_uninstall)
            $result &= Configuration::deleteByName($key_to_uninstall);

        /* Uncomment this line if you would like to also delete the Transaction details table */
        $result &= Db::getInstance()->Execute('DROP TABLE `' . _DB_PREFIX_ . 'pagadito_transaction`');

        return $result && parent::uninstall();
    }

    /* Pagadito configuration section
     *
     * @return HTML page (template) to configure the Addon
     */

    public function getContent() {
        /* Loading CSS and JS files */
        // 2013-11-8 add 1.4 support
        if (isset($this->context->controller)) {
            $this->context->controller->addCSS(array($this->_path . 'css/pagaditops.css'));
            $this->context->controller->addJS(array(_PS_JS_DIR_ . 'jquery/jquery-ui-1.8.10.custom.min.js', $this->_path . 'js/pagadito.js'));
        }

        /* Update the Configuration option values depending on which form has been submitted */
        if (Tools::isSubmit('SubmitSettings'))
            $this->_saveSettings();

        // 2013-11-8 add 1.4 token support
        if (method_exists('Tools', 'getAdminTokenLite')) {
            $token = Tools::getAdminTokenLite('AdminModules');
        } else {
            $tabid = (int) Tab::getCurrentTabId();
            $employee_id = (int) $this->context->cookie->id_employee;
            $token = 'AdminModules' . $tabid . $employee_id;
            $token = Tools::getAdminToken($token);
        }
        $this->context->smarty->assign(array(
            'pagadito_tracking' => 'http://www.prestashop.com/modules/pagadito.png?url_site=' . Tools::safeOutput($_SERVER['SERVER_NAME']) . '&id_lang=' . (int) $this->context->cookie->id_lang,
            'pagadito_form_link' => './index.php?tab=AdminModules&configure=pagaditops&token=' . Tools::getAdminTokenLite('AdminModules') . '&tab_module=' . $this->tab . '&module_name=pagaditops',
            'pagadito_ssl' => Configuration::get('PS_SSL_ENABLED'),
            'pagadito_validation' => (empty($this->_validation) ? false : $this->_validation),
            'pagadito_error' => (empty($this->_error) ? false : $this->_error),
            'pagadito_warning' => (empty($this->_warning) ? false : $this->_warning),
            'pagadito_configuration' => Configuration::getMultiple(array('PAGADITO_UID', 'PAGADITO_WSK', 'PAGADITO_SANDBOX')),
            'pagadito_js_files' => stripcslashes('"' . _PS_JS_DIR_ . 'jquery/jquery-ui-1.8.10.custom.min.js","' . $this->_path . 'js/pagaditops.js' . '"')
        ));

        return $this->display(__FILE__, 'views/templates/admin/configuration.tpl');
    }

    /*
     * Pagadito configuration section - Baseic settings (API credentials, Enable pending payments options)
     */

    private function _saveSettings() {

        if (!isset($_POST['pagadito_uid']) || !$_POST['pagadito_uid'])
            $this->_error[] = $this->l('Your Pagadito UID is required.');
        if (!isset($_POST['pagadito_wsk']) || !$_POST['pagadito_wsk'])
            $this->_error[] = $this->l('Your Pagadito WSK is required.');

        Configuration::updateValue('PAGADITO_UID', pSQL(Tools::getValue('pagadito_uid')));
        Configuration::updateValue('PAGADITO_WSK', pSQL(Tools::getValue('pagadito_wsk')));
        Configuration::updateValue('PAGADITO_SANDBOX', (bool) Tools::getValue('pagadito_sandbox'));

        if (!count($this->_error))
            $this->_validation[] = $this->l('Congratulations, your configuration was updated successfully');
    }

    /* Pagadito payment hook
     *
     * @param $params Array Default PrestaShop parameters sent to the hookPaymentOptions() method (Order details, etc.)
     *
     * @return HTML content (Template) displaying the enable Pagadito payment method
     */

    public function hookPaymentOptions($params) {

        if (!$this->active) {
            return false;
        }

        $currency = new Currency((int) $params['cart']->id_currency);
        if (!$this->active || Configuration::get('PAGADITO_UID') == '' || Configuration::get('PAGADITO_WSK') == '' || !in_array($currency->iso_code, array('USD', 'GTQ', 'HNL', 'NIO', 'CRC', 'PAB', 'DOP'))) {
            return false;
        }

        $payments_options = array (
            $this->getExternalPaymentOption(),
        );

        return $payments_options;
    }

    /**
     * This method create the element PaymentOption and allow show The Pagadito payment method in the frontend
     * @return object PaymentOption
     */
    public function getExternalPaymentOption()
    {
        $externalOption = new PaymentOption();
        $externalOption->setCallToActionText($this->l('Pay with Pagadito'))
                       ->setAction($this->context->link->getModuleLink($this->name, 'payment', array(), true))
                       ->setLogo(Media::getMediaPath(_PS_MODULE_DIR_.$this->name.'/img/pagadito.jpg'));
        return $externalOption;
    }

    /* Pagadito Admin order detail hook
     *
     * @param $params Array Default PrestaShop parameters sent to the hookAdminOrder() method
     *
     * @return HTML content (Template) displaying the Transaction details
     */

    public function hookAdminOrder($params) {
        /* Check if the order was paid with this Addon and display the Transaction details */
        if (Db::getInstance()->getValue('SELECT module FROM ' . _DB_PREFIX_ . 'orders WHERE id_order = ' . (int) $_GET['id_order']) == $this->name) {
            /* Do not display the refund block unless the API crendetials are set */
            if (Configuration::get('PAGADITO_UID') == '' || Configuration::get('PAGADITO_WSK') == '')
                return;

            /* Retrieve the transaction details */
            $pagaditops_transaction_details = Db::getInstance()->getRow('
			SELECT *
			FROM ' . _DB_PREFIX_ . 'pagadito_transaction
			WHERE id_order = ' . (int) $_GET['id_order'] . ' AND id_shop = ' . (int) $this->context->shop->id);

            $this->context->smarty->assign(array(
                'pagaditops_transaction_details' => $pagaditops_transaction_details));

            return $this->display(__FILE__, 'views/templates/admin/admin-order.tpl');
        }
    }

    /**
     * This method is required by the new version of Prestashop 1.7
     */
    public function hookPaymentReturn($params)
    {
    }

    /* Pagadito Order confirmation hook
     *
     * @param $params Array Default PrestaShop parameters sent to the hookDisplayOrderConfirmation() method
     *
     * @return HTML content (Template) displaying a confirmation or error message upon order creation
     */

    public function hookDisplayOrderConfirmation($params)
    {
        if (!isset($params['order'])) {
            return false;
        }

        if ($params['order']->module != 'pagaditops') {
            return false;
        }

        // pagadito_referencce
        $order_payment = new OrderPayment();
        $order_payment_info = $order_payment->getByOrderReference($params['order']->reference);
        $pagadito_reference = $order_payment_info[0]->transaction_id;
        
        // retrieve the pagadito transaction details */
        $pagaditops_transaction_details = Db::getInstance()->getRow("
		SELECT *
		FROM " . _DB_PREFIX_ . "pagadito_transaction
		WHERE pagadito_reference = '" . $pagadito_reference . "'"
        );
        
        $this->context->smarty->assign(
            'pagaditops_order',
            array(
            'transaction_id' => $pagadito_reference,
            'valid' => $params['order']->valid,
            'pagadito_current_state' => $pagaditops_transaction_details['pagadito_transaction_status'],
            )
        );
        return $this->context->smarty->fetch('module:pagaditops/views/templates/hook/order-confirmation.tpl');
    }

    public function validation() {
        if ($this->active) {

            if (Configuration::get('PAGADITO_UID') && Configuration::get('PAGADITO_WSK')) {
                $pagadito_token = Tools::getValue('token');
                $id_cart = (int) Tools::getValue('id_cart');
                $cart = $this->context->cart;
                if (Validate::isLoadedObject($cart) && !Order::getOrderByCartId($id_cart)) {
                    $amount = $cart->getOrderTotal(true);

                    $Pagadito_API = new Pagadito(Configuration::get('PAGADITO_UID'), Configuration::get('PAGADITO_WSK'));

                    if (Configuration::get('PAGADITO_SANDBOX')) {
                        $Pagadito_API->mode_sandbox_on();
                    }

                    if (!$Pagadito_API->connect()) {
                        // error en el token
                        die($this->module->l('connect ' . $Pagadito_API->get_rs_message(), 'validation'));
                    }
                    $pagadito_token_consulta = $Pagadito_API->get_rs_value();

                    if (!$Pagadito_API->get_status($pagadito_token)) {
                        // error en la consulta del token
                        die($this->module->l('get_status ' . $Pagadito_API->get_rs_message(), 'validation'));
                    }

                    $pagadito_response = $Pagadito_API->get_rs_status();
                    $pagadito_reference = null;
                    $order_status = null;

                    switch ($pagadito_response) {

                        case 'COMPLETED':

                            $pagadito_reference = $Pagadito_API->get_rs_reference();
                            $order_status = (int) Configuration::get('PS_OS_PAYMENT');
                            break;

                        case 'VERIFYING':

                            $pagadito_reference = $Pagadito_API->get_rs_reference();
                            $order_status = (int) Configuration::get('PAYPAL_OS_WAITING');
                            break;

                        case 'FAILED':
                        default:

                            $order_status = (int) Configuration::get('PS_OS_ERROR');
                            break;
                    }

                    $message = '
                                token: ' . $pagadito_token . '
                                id_cart: ' . $id_cart . '
                                query token: ' . $pagadito_token_consulta . '
                                response: ' . $pagadito_response . '
                                reference: ' . $pagadito_reference;

                    //$currency = new Currency((int) $cart->id_currency);
                    $currency = $this->context->currency;
                    $customer = new Customer((int) $cart->id_customer);

                    if (in_array($pagadito_response, array('COMPLETED', 'VERIFYING'))) {
                        if ($this->validateOrder($cart->id, $order_status, $amount, $this->displayName, $message, array(), (int)$currency->id, false, $customer->secure_key, null)) {
                            /* Store the transaction ID and details */
                            $this->addTransactionId((int) $this->currentOrder, $pagadito_reference);
                            $this->addTransaction(array(
                                'id_shop' => (int) $cart->id_shop,
                                'id_customer' => (int) $cart->id_customer,
                                'id_cart' => (int) $cart->id,
                                'id_order' => (int) $this->currentOrder,
                                'amount' => (float) $amount,
                                'currency' => $currency->iso_code,
                                'pagadito_token' => $pagadito_token,
                                'pagadito_transaction_status' => $pagadito_response,
                                'pagadito_reference' => $pagadito_reference
                                    )
                            );
                        }

                        Tools::redirect('index.php?controller=order-confirmation&id_cart=' . $cart->id . '&id_module=' . $this->id . '&id_order=' . (int) $this->currentOrder . '&key=' . $customer->secure_key);
                    } else {
                        Tools::redirect('index.php?controller=order');
                    }

                } else {
                    die($this->module->l('Unfortunately your order could not be validated. Error: Invalid Cart ID', 'validation'));
                }
            }
        }
    }

    /* Pagadito Order Transaction ID update
     * Attach a Pagadito Reference to an existing order (it will be displayed in the Order details section of the Back-office)
     *
     * @param $id_order integer Order ID
     * @param $id_transaction string Pagadito Reference
     */

    public function addTransactionId($id_order, $id_transaction) {
        if (version_compare(_PS_VERSION_, '1.5', '>=')) {
            $new_order = new Order((int) $id_order);
            if (Validate::isLoadedObject($new_order)) {
                $payment = $new_order->getOrderPaymentCollection();
                if (isset($payment[0])) {
                    $payment[0]->transaction_id = pSQL($id_transaction);
                    $payment[0]->save();
                }
            }
        }
    }

    /* Pagadito Transaction details update
     * Attach transactions details to an existing order (it will be displayed in the Order details section of the Back-office)
     *
     * @param $details Array Transaction details
     *
     * @return boolean Operation result
     */

    public function addTransaction($details) {
        $sandbox_value = Configuration::get('PAGADITO_SANDBOX');

        return Db::getInstance()->Execute('
		INSERT INTO ' . _DB_PREFIX_ . 'pagadito_transaction (
                    mode,
                    id_shop,
                    id_customer,
                    id_cart,
                    id_order,
                    amount,
                    currency,
                    date_add,
                    pagadito_token,
                    pagadito_transaction_status,
                    pagadito_reference
                ) VALUES (
                    \'' . ($sandbox_value ? 'sandbox' : 'live') . '\',
                    ' . (int) $details['id_shop'] . ',
                    ' . (int) $details['id_customer'] . ',
                    ' . (int) $details['id_cart'] . ',
                    ' . (int) $details['id_order'] . ',
                    \'' . (float) $details['amount'] . '\',
                    \'' . pSQL($details['currency']) . '\',
                    NOW(),
                    \'' . $details['pagadito_token'] . '\',
                    \'' . $details['pagadito_transaction_status'] . '\',
                    \'' . $details['pagadito_reference'] . '\'
                )');
    }

    public function getModuleLink($module, $controller = 'default', array $params = array(), $ssl = null) {
        $link = $this->context->link->getModuleLink($module, $controller, $params, $ssl);
        return $link;
    }

}
